@@ -62,10 +62,10 @@ with other DBs.
 
 =cut
 
-use v5.10;
+use v5.12;
 
 package SQL::Steno;
-our $VERSION = 'v0.1.2';	# quotes because of CPAN indexer bug
+our $VERSION = 'v0.2.1';	# quotes because of CPAN indexer bug
 
 use strict;
 use Time::HiRes qw(gettimeofday tv_interval);
@@ -105,7 +105,20 @@ sub init_from_query {
 }
 my @keys_Table_Columns;
 
+my %render =
+    (csv => \&render_csv,
+     yaml => \&render_yaml,
+     yml => \&render_yaml);
+my $render;
+
 our( %Queries_help, %Queries );
+{
+    my $setter = sub { $render = $render{substr $_[0], 1}; "" };
+    for my $fmt ( keys %render ) {
+	$Queries_help{".$fmt"} = "   output '&.$fmt() this' or next query as \U$fmt";
+	$Queries{".$fmt"} = $setter;
+    }
+}
 sub Queries(@) {
     for( @_ ) {
 	$Queries_help{$_->[0]} = $_->[1];
@@ -113,6 +126,10 @@ sub Queries(@) {
     }
 }
 Queries
+    ['-' => "   output next query as YAML",
+      '&.yaml'],
+
+
     [ps => '   show processlist (without Sleep)',
       '/^(?!.+\|Sleep)/=show processlist'],
 
@@ -326,6 +343,7 @@ sub convert_Queries($$) {
     local $_ =  $_[1];
     &convert_table_column;
     my @arg = split ',';
+    return &$res( $_[0], @arg ) // '' if ref $res;
     my @rest;
     for my $i ( 1..@arg ) {
 	$res =~ s/\$$i/$arg[$i - 1]/g or $res =~ s/\$"$i"/"$arg[$i - 1]"/g or push @rest, $arg[$i - 1];
@@ -368,7 +386,8 @@ the leading C<#>.  E.g. C<:ljtbl#t> might expand to C<left join table t>.
 
 Those starting with C<:gb> (group by) or C<:ob> (order by) may be followed by
 result columns numbers from 1-9, each optionally followed by a or d for asc or
-desc.
+desc.  Macros can also be C<:{Perl code}>, which gets replaced by what it
+returns.
 
 Then there are the time macros, where an optional leading letter indicates a
 base time, and an optional trailing letter with an optional count means the
@@ -497,14 +516,185 @@ sub convert {
 }
 
 
-my %ctrl =
-    ("\t", '\t',
-     "\n", '\n',
-     "\r", '\r');
-my $lasttime = time;
+# escape map for special replacement characters
+my %esc = map { $_ eq 'v' ? "\013" : eval( qq!"\\$_"! ), "\\$_" } qw'0 a b e f n r t v \ "';
 
-sub run($;$$$$) {
-    my( $sql, $re, $sub, $fh, $csv ) = @_;
+# With an argument of total number of rows, init output counting and return undef if it is to be skipped (not stdout).
+# Without an argument, do the counting and return undef if no more rows wanted.
+{
+    my( $total, $cnt, $i );
+    sub count(;$) {
+	if( @_ ) {
+	    $total = $_[0];
+	    $cnt = 0;
+	    $i = 100;
+	    return select eq 'main::STDOUT' ? 1 : undef;
+	}
+	++$cnt;
+	if( --$i <= 0 && $cnt < $total ) {
+	    printf STDERR "How many more, * for all, or q to quit? (%d of %d) [default: 100] ",
+		$cnt, $total;
+	    $i = <>;
+	    if( defined $i ) {
+		$i =~ tr/qQxX \t\n\r/0000/d;
+		$i = (0 == length $i) ? 100 :
+		    $i eq '*' ? ~0 :
+			$i == 0 ? return :
+			    $i;
+	    } else {
+		print "\n";
+		return;
+	    }
+	}
+	1;
+    }
+
+    my $esc_re = join '', sort keys %esc;
+    $esc_re =~ s!\\!\\\\!;
+    sub yaml_quote(;$) {
+	if( !defined ) {
+	    $_ = '~';
+	} elsif( /^(?:y(?:es)?|no?|true|false|o(?:n|ff)|-?\.inf|\.nan)$/s ) { # can only be string in Perl or DB
+	    $_ = "'$_'";
+	} elsif( tr/][{},?:`'"|<>&*!%#@=~\0-\010\013-\037\177-\237-// or $_[0] && tr/\n// ) {
+	    s/([$esc_re])/$esc{$1}/go;
+	    s/([\0-\010\013-\037\177-\237])/sprintf "\\x%02x", ord $1/ge;
+	    $_ = qq!"$_"!;
+	} elsif( tr/\n//d ) {
+	    my $nl = chomp;
+	    s/^/    /mg;
+	    substr $_, 0, 0, $nl ? "|2\n" : "|2-\n";
+	}
+    }
+}
+
+sub render_csv($;$$) {
+    my( $sth, $re, $sub ) = @_;
+    my $filter;			# don't filter names
+    my $name = $sth->{NAME};
+    my @row = @$name;
+    while( @row ) {
+	next if $sub && $filter && !$sub->( \@row, $name );
+	next if $re  && $filter && join( '|', map $_ // '\@', @row ) !~ $re;
+	for( @row ) {
+	    $_ //= '';
+	    $_ = qq!"$_"! if
+		s/"/""/g or
+		tr/,\n// or
+		/\A=/;
+	    utf8::decode $_;
+	}
+	print join( ',', @row ) . "\n";
+    } continue {
+	@row = $sth->fetchrow_array;
+	$filter = 1;
+    }
+}
+
+sub render_tabular($;$$) {
+    my( $sth, $re, $sub ) = @_;
+    my $filter;			# don't filter names
+    my @name = @{$sth->{NAME}};
+    my @len = (0) x @name;
+    my( @txt, @res, @comp );
+    while( my @res1 = $sth->fetchrow_array ) {
+	next if $sub && $filter && !$sub->( \@res1, \@name );
+	next if $re  && $filter && join( '|', map $_ // '\@', @res1 ) !~ $re;
+	for my $i ( 0..$#res1 ) {
+	    if( !defined $res1[$i] ) {
+		$res1[$i] = '\@';
+	    } elsif( $res1[$i] !~ /^\d+(?:\.\d+)?$/ ) {
+		$txt[$i] = 1;
+		$res1[$i] =~ s/\r\n/\\R/g;
+		$res1[$i] =~ s/([\t\n\r])/$esc{$1}/g;
+		no warnings 'uninitialized';
+		$res1[$i] =~ s/^(?:(0000-)00-00|(1970-)01-01)(?:( 00:)00:00)?$/$1$2$3/ or
+		    $res1[$i] =~ s/^(\d{4}-\d\d-\d\d )?(?:00:00:00|23:59:5(9))$/$1 . ($2 ? '24:' : '00:')/e or
+			$res1[$i] =~ s/^((\d{4}-\d\d-\d\d )?\d\d:\d\d):00$/$1/;
+		utf8::decode $res1[$i];
+	    }
+	    $txt[$i] = 0 if @txt < $i;
+	    my $len = length $res1[$i];
+	    $len[$i] = $len if $len[$i] < $len;
+	}
+	if( @comp ) {
+	    for my $i ( 0..$#comp ) {
+		undef $comp[$i] if defined $comp[$i] && $comp[$i] ne $res1[$i];
+	    }
+	} else {
+	    @comp = @res1;
+	}
+	push @res, \@res1;
+    } continue {
+	$filter = 1;
+    }
+    if( @res ) {
+	@comp = () if @res == 1;
+	my $fmt = '';
+	for( my $i = 0; $i < @name; ++$i ) {
+	    $name[$i] =~ s/\r\n/\\R/g;
+	    $name[$i] =~ s/([\t\n\r])/$esc{$1}/g;
+	    if( defined $comp[$i] ) {
+		my $more;
+		while( defined $comp[$i] ) {
+		    printf $fmt, @name[0..$i-1] unless $more;
+		    $more = 1;
+		    printf "[%s=%s]", $name[$i], $comp[$i];
+		    @name[0..$i] = ('') x ($i+1);
+		    for my $row ( \@comp, \@name, \@len, \@txt, @res ) {
+			splice @$row, $i, 1;
+		    }
+		}
+		print "\n";
+		--$i, next;
+	    }
+	    if( $len[$i] < length $name[$i] ) {
+		printf "$fmt%s\n", @name[0..$i];
+		@name[0..$i] = ('') x ($i+1);
+	    }
+	    $fmt .= '%' . ($txt[$i] ? -$len[$i] : $len[$i]) . 's|';
+	}
+	$fmt .= "\n";
+	printf $fmt, @name if $name[-1];
+	printf $fmt, map '-'x$_, @len;
+	my $count = count @res;		# init
+	for my $row ( @res ) {
+	    printf $fmt, @$row;
+	    defined count or last if defined $count;
+	}
+    }
+}
+
+sub render_yaml($;$$) {
+    my( $sth, $re, $sub ) = @_;
+    my @label = my @name = @{$sth->{NAME}};
+    my $maxlen = 0;
+    for( @label ) {
+	yaml_quote 1;
+	substr $_, 0, 0, $maxlen ? '  ' : '- ';
+	$maxlen = length if $maxlen < length;
+    }
+    $_ .= ': ' . ' ' x ($maxlen - length)
+	for @label;
+    while( my @res = $sth->fetchrow_array ) {
+	next if $sub && !$sub->( \@res, \@name );
+	next if $re  && join( '|', map $_ // '\@', @res ) !~ $re;
+	my $count = count @res;		# init
+	my $i;
+	local $_;
+	while( ($i, $_) = each @res ) {
+	    yaml_quote;
+	    printf "$label[$i]$_\n";
+	    defined count or last if defined $count;
+	}
+    }
+}
+
+
+
+my $lasttime = time;
+sub run($;$$) {
+    my( $sql, $re, $sub ) = @_;
     my $t0 = [gettimeofday];
     if( $DBI::err || $t0->[0] - $lasttime > 3600 and !$dbh->ping ) {
 	printf STDOUT "Inactive for %ds, ping failed after %.03fs, your session variables are lost.\n",
@@ -517,106 +707,13 @@ sub run($;$$$$) {
     if( my $sth = UNIVERSAL::isa( $sql, 'DBI::st' ) ? $sql : $dbh->prepare( $sql )) {
 	my $t1 = [gettimeofday];
 	$sth->execute;
-	# \todo Steno time, optionally #rows grepped
 	printf STDOUT "prepare: %.03fs   execute: %.03fs   rows: %d\n",
 	    tv_interval( $t0, $t1 ), tv_interval( $t1 ), $DBI::rows;
 	if( $sth->{Active} ) {
-	    my @name = @{$sth->{NAME}};
-	    print join( ',', @name ) . "\n" if $csv;
-	    my @len = (1) x @name;
-	    my( @txt, @res, @comp );
-	    while( my @res1 = $sth->fetchrow_array ) {
-		next if $sub && !$sub->( \@res1, \@name );
-		next if $re && join( '|', map $_ // '\@', @res1 ) !~ $re;
-		if( $csv ) {
-		    for( @res1 ) {
-			$_ //= '';
-			$_ = qq!"$_"! if
-			    s/"/""/g or
-			    tr/,\n// or
-			    /\A=/;
-			utf8::decode $_;
-		    }
-		    print join( ',', @res1 ) . "\n";
-		    next;
-		} else {
-		    for my $i ( 0..$#res1 ) {
-			if( !defined $res1[$i] ) {
-			    $res1[$i] = '\@';
-			} elsif( $res1[$i] !~ /^\d+(?:\.\d+)?$/ ) {
-			    $txt[$i] = 1;
-			    $res1[$i] =~ s/\r\n/\\R/g;
-			    $res1[$i] =~ s/([\t\n\r])/$ctrl{$1}/g;
-			    no warnings 'uninitialized';
-			    $res1[$i] =~ s/^(?:(0000-)00-00|(1970-)01-01)(?:( 00:)00:00)?$/$1$2$3/ or
-				$res1[$i] =~ s/^(\d{4}-\d\d-\d\d )?(?:00:00:00|23:59:5(9))$/$1 . ($2 ? '24:' : '00:')/e or
-				$res1[$i] =~ s/^((\d{4}-\d\d-\d\d )?\d\d:\d\d):00$/$1/;
-			    utf8::decode $res1[$i];
-			}
-			$txt[$i] = 0 if @txt < $i;
-			my $len = length $res1[$i];
-			$len[$i] = $len if $len[$i] < $len;
-		    }
-		    if( @comp ) {
-			for my $i ( 0..$#comp ) {
-			    undef $comp[$i] if defined $comp[$i] && $comp[$i] ne $res1[$i];
-			}
-		    } else {
-			@comp = @res1;
-		    }
-		    push @res, \@res1;
-		}
-	    }
-	    if( @res ) {
-		@comp = () if @res == 1;
-		my $fmt = '';
-		for( my $i = 0; $i < @name; ++$i ) {
-		    $name[$i] =~ s/\r\n/\\R/g;
-		    $name[$i] =~ s/([\t\n\r])/$ctrl{$1}/g;
-		    if( defined $comp[$i] ) {
-			my $more;
-			while( defined $comp[$i] ) {
-			    printf $fmt, @name[0..$i-1] unless $more;
-			    $more = 1;
-			    printf "[%s=%s]", $name[$i], $comp[$i];
-			    @name[0..$i] = ('') x ($i+1);
-			    for my $row ( \@comp, \@name, \@len, \@txt, @res ) {
-				splice @$row, $i, 1;
-			    }
-			}
-			print "\n";
-			--$i, next;
-		    }
-		    if( $len[$i] < length $name[$i] ) {
-			printf "$fmt%s\n", @name[0..$i];
-			@name[0..$i] = ('') x ($i+1);
-		    }
-		    $fmt .= '%' . ($txt[$i] ? -$len[$i] : $len[$i]) . 's|';
-		}
-		$fmt .= "\n";
-		printf $fmt, @name if $name[-1];
-		printf $fmt, map '-'x$_, @len;
-		my $cnt = 0;
-		my $i = 100;
-		for my $row ( @res ) {
-		    printf $fmt, @$row;
-		    ++$cnt;
-		    if( !$fh && --$i <= 0 && $cnt < @res ) {
-			printf STDERR "How many more, or * for all? (%d of %d) [default: 100] ",
-			    $cnt, scalar @res;
-			$i = <>;
-			if( defined $i ) {
-			    $i =~ tr/ \t\n\r//d;
-			    $i = (0 == length $i) ? 100 :
-				$i eq '*' ? ~0 :
-				$i == 0 ? last :
-				$i;
-			} else {
-			    print "\n";
-			    last;
-			}
-		    }
-		}
+	    if( $render ) {
+		&$render( $sth, $re, $sub );
+	    } else {
+		render_tabular $sth, $re, $sub;
 	    }
 	}
     }
@@ -692,7 +789,7 @@ Run it.
 =item >>filename
 
 Redirect or append next statement's output to I<filename>.  If it has a suffix
-C<.csv>, an according format is used.
+C<.csv>, C<.yaml> or C<.yml>, an according format is used.
 
 =item |System Shell code
 
@@ -705,7 +802,7 @@ Pipe next statement's output through I<System Shell code>.
 our $prompt = 'steno> ';
 sub shell() {
     print STDERR $prompt;
-    my( $fh, $csv );
+    my $fh;
     while( <> ) {
 	undef $error;
 	goto NEXT unless /\S/;
@@ -725,8 +822,8 @@ sub shell() {
 
 	until( $error ) {
 	    last unless
-		s!^\&(\w+)(\(((?:(?>[^()]+)|(?2))*)\))!convert_Queries $1, $3!e
-		or s!^\&(\w+) *(.*)!convert_Queries $1, $2!e
+		s!^\&(\.?\w+|-)(\(((?:(?>[^()]+)|(?2))*)\))!convert_Queries $1, $3!e
+		or s!^\&(\.?\w+|-) *(.*)!convert_Queries $1, $2!e
 		or my $perl = s!^\&$perl_re!!;
 	    if( $perl ) {
 		$perl = eval $1;
@@ -775,8 +872,8 @@ sub shell() {
 	    goto NEXT;
 	}
 	s/^\s*()//;			# dummy because $1 survives loop iterations :-o
-	if( /\A(>{1,2})\s*(.+?(\.csv)?)\s*\Z/ ) {	# redirect output
-	    $csv = 1 if $3;
+	if( /\A(>{1,2})\s*(.+?(?:\.(\w+))?)\s*\Z/ ) {	# redirect output
+	    $render = $render{$3} if $3;
 	    open $fh, "$1:utf8", (glob $2)[0];
 	    select $fh;
 	    goto NEXT;
@@ -794,12 +891,12 @@ sub shell() {
 	goto NEXT if $skip;
 
       RUN:
-	run $_, $re, $sub, $fh, $csv if $dbh;
+	run $_, $re, $sub if $dbh;
+	undef $render;
 	if( $fh ) {
 	    close;
 	    select STDOUT;
 	    undef $fh;
-	    undef $csv;
 	}
       NEXT:
 	print STDERR $prompt;
@@ -865,7 +962,7 @@ The fields joined with '|' are grepped if regexp is given, case-insensitively if
 
 ??query		Only shows massaged query.
 !perl-code	Runs perl-code.
->file		Next query's output to file.  In csv format if filename has .csv suffix.
+>file		Next query's output to file.  In csv or yaml format if filename has that suffix.
 
 Query has the form {select|update|insert|delete}{fieldlist};tablelist{;clause} or set ...
 'select' is prepended if none of these initial keywords.
@@ -875,10 +972,12 @@ fieldlist defaults to '*', also if Query starts with '#'.
 Abbreviations, more help with ?&{abbrev}, ?#{abbrev}, ?.{abbrev}, ?{abbrev}(, ?:{abbrev}
 &query $1,$2,...	# only at bol
 &query($1,$2,...)...	# only at bol, only replace upto )
+&{Perl code}...		# only at bol, if it return undef then skip, else prepend to ...
 #table
 .column			# for any table recognized in statement
 function(
 :macro
+:{Perl code}		# dynamic macro
 
 Characters \t\n\r get masked in output, \r\n as \R.
 Date or time 0000-00-00 -> 0000-  1970-01-01 -> 1970-  00:00:00 -> 00:  23:59:59 -> 24:
@@ -903,7 +1002,7 @@ HELP
     package SQL::Steno;		# doesn't export yet, so get the functions easily
     use SQL::Steno;
     use DBI;
-    our $sql = DBI->connect( ... ); # preferably mysql, but other DBs should work (with limitations).
+    our $dbh = DBI->connect( ... ); # preferably mysql, but other DBs should work (with limitations).
     # If you want #tbl and .col to work, (only) one of:
     init_from_query;		# fast, defaults to mysql information_schema, for which you need read permission
     init;			# slow, using DBI dbh methods.
